Attribute VB_Name = "modFunctions"
Option Explicit

Public FlagMultipleDevicesDetected, FlagCreateCollection, FlagForceCollectionCreation As Boolean

Public Function GetWMIServices()
    Set GetWMIServices = GetObject("winmgmts:" _
        & "{impersonationLevel=impersonate}!\\gabi\root\cimv2")
End Function

Public Function WMIDateStringToDate(dtmWMIDate)
    If Not IsNull(dtmWMIDate) Then
    WMIDateStringToDate = CDate(Mid(dtmWMIDate, 5, 2) & "/" & _
         Mid(dtmWMIDate, 7, 2) & "/" & Left(dtmWMIDate, 4) & _
         " " & Mid(dtmWMIDate, 9, 2) & ":" & _
         Mid(dtmWMIDate, 11, 2) & ":" & Mid(dtmWMIDate, _
         13, 2))
    End If
End Function

Private Function GetCIMType(iX As Integer) As String
    Select Case iX
        Case wbemCimtypeBoolean: GetCIMType = "Boolean"
        Case wbemCimtypeChar16: GetCIMType = "String(16)"
        Case wbemCimtypeDatetime: GetCIMType = "Date-Time"
        Case wbemCimtypeObject: GetCIMType = "Object (Variant)"
        Case wbemCimtypeReal32: GetCIMType = "Real 32"
        Case wbemCimtypeReal64: GetCIMType = "Real 64"
        Case wbemCimtypeReference: GetCIMType = "Reference"
        Case wbemCimtypeSint16: GetCIMType = "Sint16"
        Case wbemCimtypeSint32: GetCIMType = "Sint32"
        Case wbemCimtypeSint64: GetCIMType = "Sint64"
        Case wbemCimtypeSint8: GetCIMType = "Sint8"
        Case wbemCimtypeString: GetCIMType = "String"
        Case wbemCimtypeUint16: GetCIMType = "Uint16"
        Case wbemCimtypeUint32: GetCIMType = "Uint32"
        Case wbemCimtypeUint64: GetCIMType = "Uint64"
        Case wbemCimtypeUint8: GetCIMType = "Uint8"
    End Select
End Function
Public Function GetCIMTypeVB(iX As Integer) As String
    Select Case iX
        Case wbemCimtypeBoolean: GetCIMTypeVB = "Boolean"
        Case wbemCimtypeChar16: GetCIMTypeVB = "String"
        Case wbemCimtypeDatetime: GetCIMTypeVB = "String"
        Case wbemCimtypeObject: GetCIMTypeVB = "Variant"
        Case wbemCimtypeReal32: GetCIMTypeVB = "Integer"
        Case wbemCimtypeReal64: GetCIMTypeVB = "Long"
        Case wbemCimtypeReference: GetCIMTypeVB = "Reference"
        Case wbemCimtypeSint16: GetCIMTypeVB = "String"
        Case wbemCimtypeSint32: GetCIMTypeVB = "String"
        Case wbemCimtypeSint64: GetCIMTypeVB = "String"
        Case wbemCimtypeSint8: GetCIMTypeVB = "String"
        Case wbemCimtypeString: GetCIMTypeVB = "String"
        Case wbemCimtypeUint16: GetCIMTypeVB = "Integer"
        Case wbemCimtypeUint32: GetCIMTypeVB = "Long"
        Case wbemCimtypeUint64: GetCIMTypeVB = "Double"
        Case wbemCimtypeUint8: GetCIMTypeVB = "Byte"
    End Select
End Function
Public Sub Main()
    Dim objWMIService As WbemScripting.SWbemServices
    Dim colItems As WbemScripting.SWbemObjectSet
    Dim objItem As WbemScripting.SWbemObject
    
    Set objWMIService = GetWMIServices()
    Set colItems = objWMIService.ExecQuery("Select * from Win32_BIOS")
    
    For Each objItem In colItems
        Dim i As WbemScripting.SWbemProperty
        Debug.Print "Count = " & objItem.Properties_.Count
        For Each i In objItem.Properties_
            Debug.Print "Name = " & i.Name
            If Not (IsObject(i.Value) Or IsArray(i.Value)) Then Debug.Print "Value = " & i.Value
            Debug.Print "Type = " & GetCIMType(i.CIMType)
            Debug.Print "Is Array = " & i.IsArray
            Debug.Print "Is Local = " & i.IsLocal
            Debug.Print "Origin = " & i.Origin
        Next i
    Next objItem
    
End Sub


Public Sub GenerateWMIClass(sClassName As String)
    If Len(sClassName) < 4 Then
        MsgBox "Illegal class name!"
        Exit Sub
    End If
    
    Dim objWMIService As WbemScripting.SWbemServices
    Dim colItems As WbemScripting.SWbemObjectSet
    Dim objItem As WbemScripting.SWbemObject
    
    Dim colList As colClasses
    Dim cItem As cClass
    Dim ff As Integer
    
    Dim ClassName, FileName As String
    ClassName = Mid$(sClassName, 7)
    
    Set objWMIService = GetWMIServices()
    Set colItems = objWMIService.ExecQuery("Select * from " & Trim(sClassName))
    
    Set colList = New colClasses
    
    On Error GoTo ExitSub
    
    If colItems.Count = 0 Then
        MsgBox "No items detected!" & vbCrLf & "Cannot create class!"
        Exit Sub
    End If
    
    FlagMultipleDevicesDetected = CBool(colItems.Count > 1)
    
    For Each objItem In colItems
        Dim i As WbemScripting.SWbemProperty
        
        For Each i In objItem.Properties_
            Set cItem = New cClass
            If i.Name = "Name" Or i.Name = "Size" Then
                cItem.ClassName = ClassName & i.Name
            Else
                cItem.ClassName = i.Name
            End If
            cItem.ClassType = GetCIMTypeVB(i.CIMType)
            cItem.IsClassArray = i.IsArray
            
            colList.Add cItem.ClassName, cItem.IsClassArray, cItem.ClassType
            Set cItem = Nothing
        Next i
        Exit For
    Next objItem
    
    If FlagMultipleDevicesDetected Then
        If Not FlagForceCollectionCreation Then
            ff = MsgBox("Multiple items detected on your system!" & _
                    vbCrLf & "Would you like to create the collection class also?", _
                    vbYesNo, "Create collection?")
            FlagCreateCollection = (ff = vbYes)
        End If
    End If
        
    ff = FreeFile
    FileName = App.Path & "\Classes\c" & ClassName & ".cls"
    
    On Error GoTo 0
    If Dir(App.Path & "\Classes", vbDirectory) = "" Then
        MkDir App.Path & "\Classes"
    End If
    
    
    Open FileName For Output As #ff
    Print #ff, "VERSION 1.0 CLASS"
    Print #ff, "BEGIN"
    Print #ff, "   MultiUse = -1  'True"
    Print #ff, "   Persistable = 0  'NotPersistable"
    Print #ff, "   DataBindingBehavior = 0  'vbNone"
    Print #ff, "   DataSourceBehavior = 0   'vbNone"
    Print #ff, "   MTSTransactionMode = 0   'NotAnMTSObject"
    Print #ff, "End"
    Print #ff, ""
    Print #ff, "Attribute VB_Name = " & Chr(34) & "c" & ClassName & Chr(34)
    Print #ff, "Attribute VB_GlobalNameSpace = False"
    Print #ff, "Attribute VB_Creatable = True"
    Print #ff, "Attribute VB_PredeclaredId = False"
    Print #ff, "Attribute VB_Exposed = False"
    Print #ff, "Attribute VB_Ext_KEY = " & Chr(34) & "SavedWithClassBuilder6" & Chr(34) & " ," & Chr(34) & "Yes" & Chr(34)
    Print #ff, "Attribute VB_Ext_KEY = " & Chr(34) & "Top_Level" & Chr(34) & " ," & Chr(34) & "Yes" & Chr(34)
       
    Print #ff, "Option Explicit"
    Print #ff, ""
    Print #ff, "'local variables declaration"
    
    For Each cItem In colList
        If cItem.IsClassArray Then
            Print #ff, "Private mvar" & cItem.ClassName & " As Variant"
        Else
            Print #ff, "Private mvar" & cItem.ClassName & " As " & cItem.ClassType
        End If
    Next cItem
    
    Print #ff, ""
    Print #ff, "'start writing class properties"
    Print #ff, ""
    
    
    For Each cItem In colList
        If cItem.IsClassArray Then
            'arrays
            Print #ff, "Public Property Get " & cItem.ClassName & "() As Variant"
            Print #ff, vbTab & cItem.ClassName & " = mvar" & cItem.ClassName
            Print #ff, "End Property"
            Print #ff, ""
            Print #ff, "Public Property Let " & cItem.ClassName & " (vData As Variant)"
            Print #ff, vbTab & "mvar" & cItem.ClassName & " = " & cItem.ClassName
            Print #ff, "End Property"
            Print #ff, ""
        Else
            'usual types
            Print #ff, "Public Property Get " & cItem.ClassName & "() As " & cItem.ClassType
            Print #ff, vbTab & cItem.ClassName & " = mvar" & cItem.ClassName
            Print #ff, "End Property"
            Print #ff, ""
            
            Print #ff, "Public Property Let " & cItem.ClassName & " (vData As " & cItem.ClassType & ")"
            Print #ff, vbTab & "mvar" & cItem.ClassName & " = " & cItem.ClassName
            Print #ff, "End Property"
            Print #ff, ""
        End If
    Next cItem
    
    Close #ff
    
    If FlagCreateCollection Or FlagForceCollectionCreation Then
        ff = FreeFile
        FileName = "Classes\col" & ClassName & ".cls"
        Open FileName For Output As #ff
        
        Print #ff, "VERSION 1.0 CLASS"
        Print #ff, "BEGIN"
        Print #ff, "   MultiUse = -1  'True"
        Print #ff, "   Persistable = 0  'NotPersistable"
        Print #ff, "   DataBindingBehavior = 0  'vbNone"
        Print #ff, "   DataSourceBehavior = 0   'vbNone"
        Print #ff, "   MTSTransactionMode = 0   'NotAnMTSObject"
        Print #ff, "End"
        Print #ff, ""
        Print #ff, "Attribute VB_Name = " & Chr(34) & "col" & ClassName & Chr(34)
        Print #ff, "Attribute VB_GlobalNameSpace = False"
        Print #ff, "Attribute VB_Creatable = True"
        Print #ff, "Attribute VB_PredeclaredId = False"
        Print #ff, "Attribute VB_Exposed = False"
        Print #ff, "Attribute VB_Ext_KEY = " & Chr(34) & "SavedWithClassBuilder6" & Chr(34) & " ," & Chr(34) & "Yes" & Chr(34)
        Print #ff, "Attribute VB_Ext_KEY = " & Chr(34) & "Collection" & Chr(34) & " ," & Chr(34) & "c" & ClassName & Chr(34)
        Print #ff, "Attribute VB_Ext_KEY = " & Chr(34) & "Member0" & Chr(34) & " ," & Chr(34) & "c" & ClassName & Chr(34)
        Print #ff, "Attribute VB_Ext_KEY = " & Chr(34) & "Top_Level" & Chr(34) & " ," & Chr(34) & "Yes" & Chr(34)
        
        Print #ff, "Option Explicit"
        Print #ff, ""
        
        Print #ff, "'local variable to hold collection"
        Print #ff, "Private mCol As Collection"
        Print #ff, ""
        
        Dim tmpStr As String
        Dim tmpArray
        Dim aX, eX As Integer
        
        For Each cItem In colList
            tmpStr = tmpStr & cItem.ClassName & " As " & cItem.ClassType & ","
        Next cItem
        
        eX = InStrRev(tmpStr, ",")
        tmpStr = Mid$(tmpStr, 1, eX - 1)
        
        Print #ff, "Public Function Add(Key As String," & tmpStr & ") As c" & ClassName
        
        Print #ff, "    'create a new object"
        Print #ff, "    Dim objNewMember As c" & ClassName
        Print #ff, "    Set objNewMember = New c" & ClassName
        Print #ff, ""
        Print #ff, "    'set the properties passed into the method"
        Print #ff, "     objNewMember.Key = Key"
        
        For Each cItem In colList
            Print #ff, "     objNewMember." & cItem.ClassName & " = " & cItem.ClassName
        Next cItem
        
        Print #ff, ""
        Print #ff, "     If Len(sKey) = 0 Then"
        Print #ff, "         mCol.Add objNewMember"
        Print #ff, "     Else"
        Print #ff, "         mCol.Add objNewMember, sKey"
        Print #ff, "     End If"
        Print #ff, "     'return the object created"
        Print #ff, "     Set Add = objNewMember"
        Print #ff, "     Set objNewMember = Nothing"
        Print #ff, "     End Function"
        Print #ff, ""
        
        Print #ff, "Public Property Get Item(vntIndexKey As Variant) As c" & ClassName
        Print #ff, "Attribute Item.VB_UserMemId = 0"
        Print #ff, "    'used when referencing an element in the collection"
        Print #ff, "    'vntIndexKey contains either the Index or Key to the collection,"
        Print #ff, "    'this is why it is declared as a Variant"
        Print #ff, "    'Syntax: Set foo = x.Item(xyz) or Set foo = x.Item(5)"
        Print #ff, "    Set Item = mCol(vntIndexKey)"
        Print #ff, "End Property"
        Print #ff, ""
        
        Print #ff, "Public Property Get Count() As Long"
        Print #ff, "  'used when retrieving the number of elements in the"
        Print #ff, "  'collection. Syntax: Debug.Print x.Count"
        Print #ff, "  Count = mCol.Count"
        Print #ff, "End Property"
        Print #ff, ""
        
        Print #ff, "Public Sub Remove(vntIndexKey As Variant)"
        Print #ff, "  'used when removing an element from the collection"
        Print #ff, "  'vntIndexKey contains either the Index or Key, which is why"
        Print #ff, "  'it is declared as a Variant"
        Print #ff, "  'Syntax: x.Remove(xyz)"
        Print #ff, ""
        Print #ff, "  mCol.Remove vntIndexKey"
        Print #ff, "End Sub"
        Print #ff, ""
        
        Print #ff, "Public Property Get NewEnum() As IUnknown"
        Print #ff, "  'this property allows you to enumerate"
        Print #ff, "  'this collection with the For...Each syntax"
        Print #ff, "     Set NewEnum = mCol.[_NewEnum]"
        Print #ff, "End Property"
        Print #ff, ""
        
        Print #ff, "Private Sub Class_Initialize()"
        Print #ff, "  'creates the collection when this class is created"
        Print #ff, "  Set mCol = New Collection"
        Print #ff, "End Sub"
        Print #ff, ""
        
        Print #ff, "Private Sub Class_Terminate()"
        Print #ff, "  'destroys collection when this class is terminated"
        Print #ff, "  Set mCol = Nothing"
        Print #ff, "End Sub"
        
        Close #ff
        'Debug.Print tmpStr
    End If
    
ExitSub:
    Set objWMIService = Nothing
    Set colItems = Nothing
    Set colList = Nothing
    
End Sub
